<?php

namespace Drupal\bee_hotel;

use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Provides route responses for BeeHotel module.
 */
class BeeHotelGuestMessageTokens {

  use StringTranslationTrait;

  /**
   * Get Tokens.
   */
  public function get($commerce_order) {

    $d = [];
    $d['commerce_order'] = $commerce_order;
    $d['tokens'] = $this->guestMessageTokensSchema();


    // dump ($d);
    // exit;

    foreach ($d['tokens'] as $id => $token) {

      $d['markup'] = "";

      if ($token["entity"] == "field_token") {

      }
      elseif ($token['entity'] == "commerce_order") {
        $tmp = $token['property'];
        $d['tokens'][$id]['value'] = $d['commerce_order']->get($token['field'])->$tmp;

        // @todo improve ugly is_object check.
        if (!is_object($d['tokens'][$id]['value']) ) {
          if ($tmp == "number")    {
            $d['tokens'][$id]['value'] = bee_hotel_number_format((float) $d['tokens'][$id]['value']);
          }
        }

        // Token UID.
        if (is_object($d['tokens'][$id]['value']) &&  $token['field'] == 'uid') {
          $d['tokens'][$id]['value'] = $d['tokens'][$id]['value']->get("name")->value;
        }

        // Token ORDER ITEMS.
        elseif (is_object($d['tokens'][$id]['value']) && $token['field'] == 'order_items') {
          $d['tokens'][$id]['value'] = $d['tokens'][$id]['value']->get("title")->value;

          // @todo Get BAT booking record
          $d['order_first_item'] = $d['commerce_order']->get('order_items')[0]->entity;
          if ($d['order_first_item']->hasField('field_booking')) {
            $d['order_first_item_booking'] = $d['order_first_item']->get('field_booking')->entity;
            $d['order_first_item_booking_event'] = $d['order_first_item_booking']->get('booking_event_reference')->entity;

            if (isset($d['order_first_item_booking_event'])) {

              $d['order_id'] = $d['commerce_order']->get('order_id')->value;
              $d['order_first_item_booking_event_id'] = $d['order_first_item_booking_event']->get('id')->value;
              $d['markup'] .= "<a href='/admin/bat/events/event/" . $d['order_first_item_booking_event_id'] . "/edit?/admin/commerce/orders/" . $d['order_id'] . "'>Event</a>";

            }
          }
        }

        // Hook override
        if ( isset($token['hook']) &&  $token['hook'] == TRUE) {

          if ($d['tokens'][$id]['value'] > 0) {
            // Move this in a separate hook-like
            // >>>>>
            if ($id == "balance_cash_euro") {

              $d['tokens'][$id]['value'] = $d['commerce_order']->get($token['field'])->$tmp;
              // subtract value from setting (@todo)
              $value_from_setting = 10;
              $d['tokens'][$id]['value'] -= $value_from_setting;
            }

            elseif ($id == "balance_cash_currencies") {

              $d['currencyapi_service'] = \Drupal::service('currencyapi.service');
              $d['all_rates'] = $d['currencyapi_service']->getAllRates();
              $d['tmp'] = "<div>You can balance also with:</div>";

              foreach ($d['all_rates']['currencyapi'] as $code => $value) {
                if ($value['is_default'] == FALSE) {
                  $d['balance_full'] = $d['commerce_order']->get($d['tokens']['balance_full']['field'])->number;
                  $d['convert_into'][$value['code']] = $d['currencyapi_service']->convert($d['balance_full'], "EUR", $value['code']);
                  $d['tmp']  .= "* " . $d['convert_into'][$value['code']] . $code . "\r";
                }
              }

              $d['tokens'][$id]['value'] = $d['tmp'];

              // @todo output via template.
              // $d['tokens'][$id]['value'] = [
              //   '#prefix' => t("Drupal stadard tokens are also supported."),
              //   '#theme' => 'bbeehotel_guest_messages_tokens_othercurrencies',
              //   'list' => $d['tmp'],
              // ];

            }
          }
        }
      }
    }
    return $d['tokens'];
  }

  /**
   * Consume token contributed module.
   */
  private function getTokenFromTokenModule($beehotel_token, $options) {
    $d['tokens'] = \Drupal::service('token.tree_builder')->buildRenderable([
      'node',
      'user',
      'commerce_order',
    ]);
    return $token;
  }

  /**
   * Structured list of BeeHotel Guest message tokens.
   */
  public function guestMessageTokensSchema() {

    $schema = [
      'balance_full' => [
        'description' => $this->t('The balance left to paid'),
        'entity' => 'commerce_order',
        'field' => 'balance',
        'property' => 'number',
        'value' => NULL,
      ],

      'balance_cash_euro' => [
        'description' => $this->t('The balance left to paid'),
        'entity' => 'commerce_order',
        'field' => 'balance',
        'property' => 'number',
        'hook' => TRUE,  // search for the overiding hook method
        'value' => NULL,
      ],

      'balance_cash_currencies' => [
        'description' => $this->t('List of currency balances by cash'),
        'entity' => 'commerce_order',
        'field' => 'balance',
        'property' => 'number',
        'hook' => TRUE,  // search for the overiding hook method
        'value' => NULL,
      ],

      'checkin_time' => [
        'description' => $this->t('Time of check IN'),
        'entity' => '',
        'field' => '',
        'property' => NULL,
        'value' => '15:00',
      ],

      'checkout_time' => [
        'description' => $this->t('Time of check OUT'),
        'entity' => '',
        'field' => '',
        'value' => '10:00',
      ],

      'guest_email' => [
        'description' => $this->t('Email used by Guest at the reservation'),
        'entity' => 'commerce_order',
        'field' => 'mail',
        'property' => 'value',
        'value' => NULL,
      ],

      'guest_name' => [
        'description' => $this->t('Guest name (or email if not provided)'),
        'entity' => 'commerce_order',
        'field' => 'mail',
        'property' => 'value',
        'value' => NULL,
      ],

      'mail' => [
        'description' => $this->t('MAil (DUPLICATE????)'),
        'entity' => 'commerce_order',
        'field' => 'mail',
        'property' => 'value',
        'value' => NULL,
      ],

      'order_items' => [
        'description' => $this->t('List of items (Bee Hotel Units) in reservations'),
        'entity' => 'commerce_order',
        'field' => 'order_items',
        'multiple' => TRUE,
        'property' => 'entity',
        'sub_property' => "purchased_entity->entity->title->value",
        'value' => NULL,
      ],

      'total_price' => [
        'description' => $this->t('Total price for current reservation'),
        'entity' => 'commerce_order',
        'field' => 'total_price',
        'property' => 'number',
        'value' => NULL,
      ],

      'total_paid' => [
        'description' => $this->t('Total paid at today'),
        'entity' => 'commerce_order',
        'field' => 'total_paid',
        'property' => 'number',
        'value' => NULL,
      ],

      'uid' => [
        'description' => $this->t('User ID'),
        'entity' => 'commerce_order',
        'field' => 'uid',
        'property' => "entity",
        'sub_property' => "uid",
        'value' => NULL,
      ],

      'order_number' => [
        'description' => $this->t('order number #'),
        'entity' => 'commerce_order',
        'field' => 'order_number',
        'property' => "value",
        // 'sub_property' => "uid",
        'value' => NULL,
      ],


    ];

    return $schema;
  }

}
